% =========================================================================
% Name  : A function for testing adaBoost for logistic regression. 
% Author: Jakramate Bootkrajang
% Date  : 18/11/2012
% Input : a  = a weight vector of the ensemble
%       : c  = a matrix of base learners (column-wise)
%       : x  = input data
%       : y  = target variable
% Output: fprd = ensemble's prediction
%         er = misclassification error showing a progress 
%              after adding a base learner to the ensemble.
% =========================================================================
% =========================================================================
function [fprd er] = gBoostClassify(a, c, x, y)

% the function uses {-1,1} class label
y  = castLabel(y,-1);
er = zeros(1,length(a));

% Combining the predictions
if ~iscell(c)
    x = addbias(x);
    % for rLR
    sum_x = zeros(size(x,1),1);
    for i = 1:size(c,2)
        yHat  = sign(x * c(:,i));
        sum_x = sum_x + a(i) * yHat;
        fprd  = sign(sum_x);
        % keeping track of performance improvement
        er(i) = sum(y ~= fprd)/size(x,1);
    end    
else
    x = x';
    y = y';
    % for tree
    sum_x = zeros(1, size(x, 2));    
    for i = 1:size(a,2);
        lrn_out = calc_output(c{i}, x);
        lrn_out = castLabel(lrn_out,-1);
        sum_x   = sum_x + lrn_out * a(i);
        fprd    = sign(sum_x);
        er(i)   = sum(y ~= fprd)/size(x,2);
    end
end
