function [h] = logitBoost(x,y,options)

y  = castLabel(y,0);

h  = [];
Fx = zeros(size(x,1),1);
p  = ones(size(x,1),1).* 0.5; % initial uniform probability for all example
t  = 0;

gamma = options.gamma;

c1 = 1-gamma(1,2);
c2 = 1-gamma(2,1);
gm = (c1 + c2)/2;
nt = 0.5*log(c2/c1);

while t < options.MaxIter % stop condition
    t  =  t+1;                % instantiate a new weak learner
    z  =  (y-p)./(p.*(1-p));  % Compute the work response values
    w  =  p.*(1-p);           % Compute the weights
    
    % --- control
    z(isnan(z))  = 0;
    z(z == inf)  = 0;
    z(z == -inf) = 0;
    % --- threashold ratio
    z(z > 4)     = 4;
    z(z < -4)    = -4;
    w(w < 2*eps) = 2*eps;
    
%     % subset resampling for creating diversity
%     perm   = randperm(size(x,1));
%     lim    = floor(size(x,1) * 0.7);
%     X      = x(perm(1:lim),:);
%     Z      = z(perm(1:lim));
%     W      = w(perm(1:lim));
    
    h(:,t) =  lscov(addbias(full(x)),z,w);
    fx     =  addbias(full(x)) * h(:,t);
        
    Fx     =  Fx + 0.5/(2-gamma(1,2)-gamma(2,1)) .* fx;  % build final classification
    
    F_num  =  exp(gm*Fx + nt);   % numerator of F     
    % --- control
    idx = F_num == inf;
    if any(idx == 1)
        F_num(idx) = realmax;
    end
    
    F_den  =  (exp(gm*Fx + nt)+exp(-gm*Fx - nt)); % denominator of F
    % --- control
    idx = F_den == inf;
    if any(idx == 1)
        F_den(idx) = realmax;
    end
    
    p        = F_num./F_den; % probability of x being in class 1
%     p0       = 1 - p1;
%     p        = gamma(2,2)*p1 + gamma(1,2)*p0;
    p(p < 0) = 0;
    
    
    %% visualising the boosting process
    if options.verbose
        % (1) plotting the data set
        gridRange = -1:0.1:1;
        subplot(1,2,1);
        plot(x(y==1,1),x(y==1,2),'kx',x(y==0,1),x(y==0,2),'ro'); hold on; drawnow
        
        % (2) plotting the decision boundary of the current learner
        [x1,x2] = meshgrid(gridRange, gridRange);
        xx      = [reshape(x1,size(x1,1)^2,1) reshape(x2,size(x2,1)^2,1)];
        yHat    = sign(addbias(xx) * h(:,t));
        subplot(1,2,2);
        if t>1; delete(hd); end;
        [Cd hd] = contour(gridRange,gridRange,reshape(yHat,size(x1,1),size(x1,1)),1);
        set(hd, 'LineWidth', 2, 'Color', [.0 .4 1]); drawnow
        
        % (3) plotting the decision boundary of the current ensemble
        pd    = sign(addbias(xx) * h);
        H     = pd * ones(1,t)';
        yHat  = sign(H);  % final hypothesis
        subplot(1,2,1);
        %if t>1; delete(hd); end;
        [Cd hd] = contour(gridRange,gridRange,reshape(yHat,size(x1,1),size(x1,1)),1);
        set(hd, 'LineWidth', 2, 'Color', [.0 .4 1]); drawnow
    end    
    
            
end